TASK_INTERPOLATION = "interpolation"
TASK_FILTERING = "filtering"
TASK_FINDING_GOOD_FRAME = "finding good frame"
TASK_SKELETON_ROTATION = "skeleton rotation"
TASK_RESULTS_VISUALIZATION = "results visualization"
TASK_DATA_SAVED = "data saved"

PARAM_METHOD = "Method"
PARAM_ORDER = "Order"
PARAM_CUTOFF_FREQUENCY = "Cutoff Frequency"
PARAM_SAMPLING_RATE = "Sampling Rate"
PARAM_ROTATE_DATA = "Rotate Data"
PARAM_AUTO_FIND_GOOD_FRAME = "Auto-find Good Frame"
PARAM_GOOD_FRAME = "Good Frame"

RAW_DATA_FILE_NAME = "mediapipe3dData_numFrames_numTrackedPoints_spatialXYZ.npy"
POSTPROCESSING_SETTINGS_FILE_NAME = 'postprocessing_settings.toml'
RECORDING_SETTINGS_FILE_NAME = 'recording_parameters.json'
RAW_DATA_FOLDER_NAME = "raw_data"
OUTPUT_DATA_FOLDER_NAME = "output_data"
