groupbox_stylesheet = """
QGroupBox {
    font-weight: bold;
    font-variant: small-caps;
    font-size: 17px;
    color: #006C9A;
}


"""

button_stylesheet = """
QPushButton {
    font-weight: bold;
    font-variant: small-caps;
    font-size: 15px;
    color: #3A3A3A;

}

"""

label_stylesheet = """
QLabel {
    font-weight: bold;
    font-variant: small-caps;
    font-size: 15px;
    color: #3A3A3A;

}

"""